#!/usr/bin/perl

use strict;
use warnings FATAL => 'all';
use Getopt::Long;


my $gene_id_pattern = "ENSG\\d+";
GetOptions
(
        'g|gene_id_pattern=s'      => \$gene_id_pattern,
);
my $genes_filename = shift;

die "Usage: $0 genes\n" if not defined $genes_filename;

my %genes;

open GENES, $genes_filename or die "Error: Unable to open $genes_filename: $!\n";
while (my $gene = <GENES>)
{
	chomp($gene);
	$genes{$gene} = 1;	
}
close GENES;

while (<>)
{
	my $line = $_;

	chomp;
	next if /^\@/;
	
	my @fields = split /\t/;

	my $reference = $fields[2];
	$reference =~ /($gene_id_pattern)/;
	my $gene = $1;

        if (!defined $gene) 
        {
           print STDERR "In $fields[2] could not find $gene_id_pattern got $reference\n";
        }
	elsif ($genes{$gene})
	{
		print $line;
	}
}

